/*************************************************************************
 * BTSMS
 * 
 * BTSMS is an application that provides support to connect bluetooth enabled
 * mobile phones from a PC and use them to send SMS. The SMSes can be written
 * over PC using the comfort of the keyboard rather than using small keypad of
 * the mobile phone
 *
 * Official Website: http://bt-tools.sourceforge.net
 * 
 * -------------------------------------------------------------------------
 * 
 * Name    : btlib.h
 *
 * Purpose : Header file for BTLIB (Bluetooth wrapper library over BlueZ)
 * 
 * Authors : Shobhit Arora
 * 
 * License : GNU Public License (Version 2 or later)
 * 
 * -------------------------------------------------------------------------
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
 * CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * *************************************************************************/
#ifndef __BT_LIB_H__
#define __BT_LIB_H__

/* typedefs */
typedef unsigned int        UINT32;
typedef unsigned short int  UINT16;
typedef unsigned char       UINT8;
typedef int                 INT32;
typedef short int           INT16;
typedef char                INT8;

/* Constant Definitions */
#define NUM_DEV_MAX     25
#define BDADDR_LEN      18
#define BDNAME_LEN      128
#define INQ_LEN         8
#define MAX_SDP_REC     6
#define PROTO_L2CAP     0x0100
#define PROTO_RFCOMM    0x0003
#define BT_NUM_SRV      10

/* Error codes */
#define     BTLIB_ERR_OK                0
#define     BTLIB_ERR_FAIL              -1
#define     BTLIB_ERR_SOCK              BTLIB_ERR_OK + 1
#define     BTLIB_ERR_NO_DEV            BTLIB_ERR_OK + 2
#define     BTLIB_ERR_SDP_CONNECT       BTLIB_ERR_OK + 3
#define     BTLIB_ERR_SDP_FIND          BTLIB_ERR_OK + 4
#define     BTLIB_ERR_RFCOMM_CON        BTLIB_ERR_OK + 5

typedef struct service_info
{
    UINT16      srv_id;
    UINT8       name[BDNAME_LEN];

}BT_SRV_INFO, *BT_SRV_INFO_PTR;

typedef struct dev_info
{
    bdaddr_t    addr;
    INT8        name[BDNAME_LEN];
    UINT8       dev_class[3];
    sdp_list_t  *service_list;
    UINT16      connected;

}   BT_DEV_INFO, *BT_DEV_INFO_PTR;

typedef struct dev_list
{
    UINT16  num_dev;
    BT_DEV_INFO bt_dev_info[NUM_DEV_MAX];

} BT_DEV_LIST, *BT_DEV_LIST_PTR;

struct context
{
    int serverdone;
    int clientdone;
    char *get_name; /* Name of last get-request */
};

extern BT_DEV_LIST  bt_dev_list;

/* Function Prototypes */
INT16 btlib_dev_find(BT_DEV_LIST_PTR dev_list, UINT16 *dev_count, UINT16 *error);
void   btlib_show_devices(BT_DEV_LIST_PTR dev_list);
int btlib_sdp_browse(bdaddr_t *dev_addr, sdp_list_t **psdp_list, UINT16 *error);
void btlib_sdp_info_show(BT_DEV_LIST_PTR pdev_list, INT16 dev_index);
int btlib_rfcomm_connect(bdaddr_t *paddr, UINT16 channel, INT32 *socket_fd, UINT16 *error);
void btlib_rfcomm_close(INT32 socket);
int btlib_rfcomm_write(INT32 socketfd, INT8 *data, UINT32 len);
int btlib_rfcomm_read(INT32 socketfd, INT8 *data, UINT32 len);
INT16 btlib_get_dialup_rfcomm_port(BT_DEV_INFO_PTR pDev);
#endif
