/*************************************************************************
 * BTSMS
 * 
 * BTSMS is an application that provides support to connect bluetooth enabled
 * mobile phones from a PC and use them to send SMS. The SMSes can be written
 * over PC using the comfort of the keyboard rather than using small keypad of
 * the mobile phone
 *
 * Official Website: http://bt-tools.sourceforge.net
 *
 * -------------------------------------------------------------------------
 *
 * Name    : testapp.c
 *
 * Purpose : Source file for BTSMS application
 *
 * Authors : Shobhit Arora
 * 
 * License : GNU Public License (Version 2 or later)
 * 
 * -------------------------------------------------------------------------
 *  
 * THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
 * CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>
#include <bluetooth/sdp.h>
#include <bluetooth/sdp_lib.h>
#include <pthread.h>
#include <signal.h>
#include "btlib.h"

volatile UINT16 keep_working = 0;
pthread_t thread;
void *show_working(void *data)
{
    while(keep_working)
    {
        printf(".");fflush(NULL);
        sleep(1);
    }
}

INT32 send_at_command(INT32 connfd, INT8 *data, INT8 *msg, UINT16 len);
void show_help();
void show_prompt();
void send_sms(BT_DEV_INFO_PTR pDev);

int main()
{
    UINT16 num_devices, error_code;
    INT16 retVal;
    sdp_list_t *psdp_list, *list_cpy;
    UINT16 dev_index = 0, channel;
    INT8    cmd[24];
    int dev_num;
    BT_DEV_LIST_PTR pdevice_list = &bt_dev_list;
    system("clear");

    printf("\tBTSMS Application - Demonstrating the use of BTLIB\n");
    printf("===================================================================");
    printf("\nSearching for bluetooth devices ");fflush(NULL);
    keep_working = 1;
    pthread_create(&thread, NULL, show_working, NULL);
    retVal = btlib_dev_find(pdevice_list, &num_devices, &error_code);
    keep_working = 0;
    fflush(NULL);

    switch(error_code)
    {
        case BTLIB_ERR_SOCK:
            printf("\nSocket Error. Error Code = %d\n", error_code);
            printf("Fatal error.\n");
            printf("===================================================================\n");
            exit(0);
        case BTLIB_ERR_NO_DEV:
            printf("\nNo devices found.\n");
            printf("===================================================================\n");
            exit(0);
    };

    keep_working = 1;
    pthread_create(&thread, NULL, show_working, NULL);
    btlib_sdp_browse_all(&pdevice_list, &error_code);
    keep_working = 0;
    printf("\n");
    btlib_show_devices(pdevice_list);
    printf("\nType help for the list of commands. CTRL+C to quit the application\n");
    printf("===================================================================\n");
    show_prompt();   
    while(1)
    {
        fgets(cmd, 24, stdin);
        if(strstr(cmd, "help"))
        {
            show_help();
        }
        else if(strcmp(cmd, "\n\0") == 0)
        {
        }
        else if(strstr(cmd, "bye"))
        {
            printf("Thanks for using BTSMS.\n");
            exit(0);
        }
        else if(strstr(cmd, "lsbt"))
        {
            char *dev = strstr(cmd, ":");
            if(!dev)
            {
                btlib_show_devices(pdevice_list);
            }
            else
            {
                dev++;

                dev_num = atoi(dev);
                btlib_sdp_info_show(pdevice_list, dev_num);
            }

        }
        else if(strstr(cmd, "btsms"))
        {
            char *dev = strstr(cmd, ":");
            if(!dev)
            {
                printf("Missing parameter\n");
            }
            else
            {
                dev++;
                dev_num = atoi(dev);
                if(dev_num <=0 || dev_num > pdevice_list->num_dev)
                {
                    printf("Invalid device\n");
                }
                else
                {
                    BT_DEV_INFO_PTR device = &(pdevice_list->bt_dev_info[dev_num - 1]);
                    send_sms(device);
                }
            }
        }
        else
        {
            printf("Command not supported.\n");
        }
        show_prompt();   
    }

    return 0;
}

INT32 send_at_command(INT32 connfd, INT8 *data, INT8 *msg, UINT16 len)
{
    INT16 retVal;
    INT8  resp[128] = { 0 };
    INT8  *str_index = NULL;
    UINT32  index;

    retVal = btlib_rfcomm_write(connfd, data, len);
    if(retVal < 0)
    {
        printf("AT command send failed\n");
        return -1;
    }

    sleep(1);
    retVal = btlib_rfcomm_read(connfd, resp, sizeof(resp));
    if(retVal < 0)
    {
        printf("AT command response failed\n");
        return -1;
    }

    if(strstr(resp, ">"))
    {
        if(!msg)
            return -1;

        retVal = btlib_rfcomm_write(connfd, msg, strlen(msg));
        memset(resp, 0, sizeof(resp));
        sleep(1);
        retVal = btlib_rfcomm_read(connfd, resp, sizeof(resp));
        if(retVal < 0)
        {
            printf("AT command response failed\n");
            return -1;
        }

        /* get the index number for this message */
        str_index = strstr(resp, ":");
        while(*str_index == ' ')
            str_index++;

        str_index++;
        index = atoi(str_index);
        return index;
    }
    return 0;
}

void show_help()
{
    printf("commands:\n");
    printf("\tlsbt  usage: lsbt           - Search and list the bluetooth device\n");
    printf("\t      usage: lsbt:<dev-id>  - List the services provided by a bluetooth device\n");
    printf("\tbtsms usage: btsms:<dev-id> - Send a SMS using the selected device\n");
    printf("\tbye   usage: bye            - Quit the application\n");
    printf("\thelp  usage: help           - Display this help \n");
}

void show_prompt()
{
    printf("btsms> ");fflush(NULL);
}

void send_sms(BT_DEV_INFO_PTR pDev)
{
    INT32 retVal, channel;
    UINT16  error_code;
    INT32 connfd;
    INT8    data[128];
    INT8    number[16];
    INT8    message[64];
    INT32 len;
    INT8    index[6];

    channel = btlib_get_dialup_rfcomm_port(pDev);
    if(channel < 0)
    {
        printf("Service not supported\n");
        return;
    }

    printf("Accept the connection on the phone ..\n");
    /* connect to the device */
    retVal = btlib_rfcomm_connect(&pDev->addr, channel, &connfd, &error_code);
    if(BTLIB_ERR_OK != retVal)
    {
        printf("Connection Failed !!!, Err Code = %d\n", error_code);
        btlib_rfcomm_close(connfd);
        return;
    }

    printf("Connected to the device.\n");
    printf("Enter the mobile number: ");fflush(stdout);
    fgets(number, 16, stdin);
    printf("Enter your message [64 charachters]:");fflush(stdout);
    fgets(message, 64, stdin);

    printf("Sending Message ");fflush(stdout);
    keep_working = 1;
    pthread_create(&thread, NULL, show_working, NULL);

    memset(data, 0, sizeof(data));
    strcpy(data, "AT+CMGF=1\r\n");
    retVal = send_at_command(connfd, data, NULL, strlen(data));
    if(retVal < 0)
    {
        printf("Message sending failed\n");
        btlib_rfcomm_close(connfd);
        return;
    }
    memset(data, 0, sizeof(data));
    strcpy(data, "AT+CMGW=");
    strcat(data, "\"+91");
    strcat(data, number);
    strcat(data, "\"\r\n");

    len = strlen(message);
    message[len++] = 26;
    message[len++] ='\r';
    message[len] ='\n';

    retVal = send_at_command(connfd, data, message, strlen(data));
    if(retVal < 0)
    {
        printf("Message sending failed\n");
        btlib_rfcomm_close(connfd);
        return;
    }

    memset(data, 0, sizeof(data));
    strcpy(data, "AT+CMSS=");
    sprintf(index, "%d", retVal);
    strcat(data, index);
    strcat(data, "\r\n");

    retVal = send_at_command(connfd, data, NULL, strlen(data));
    if(retVal < 0)
    {
        printf("Message sending failed\n");
        btlib_rfcomm_close(connfd);
        return;
    }
    else
    {
        printf("\nMessage sent.\n");
    }

    keep_working = 0;
    btlib_rfcomm_close(connfd);
}
